'''
ActiveBitmap.py

A function based on wx.StaticBitmap, capable of accepting multiple bitmaps that can be cycled through using a Timer
    a quasi-animation if you will.
    It can utilise existing Gif files, either broken down into a list of the component parts, or a valid Gif filename,
     with the caveat that they should not have been optimised for size ( see below )

ActiveBitmap will utilise PIL - Pillow (Fork of the Python Imaging Library), if available, for speed purposes,
 when processing Gif files or animated WEBP files.

Based originally on the requirement to be able to flash a bitmap to denote that something relevant or of note had occurred.
or an alternative for a pulse progress dialog to denote something is happening

ActiveBitmap will accept between 1 and many bitmaps, which once activated, will display one after the other,
 using a predefined time lapse between each image.
 Upon reaching the last bitmap or defined end of range, it cycles around to the beginning of the series or start of range
  and starts again.
 This continues until stopped, or a given time period or a given number of repetitions.

 (This means GIF files can be converted or partially converted to a list (series) using wx.Image, or pass a Gif file
  and allow ActiveBitmap to convert it - See below)

Bitmaps can be inserted into the series at a given point and removed.
A sub-set of the images to be used can be defined
The animation can be restricted by time or number of repetitions

The series may also be extended by one or more new bitmaps.

Cycling through the series can be stopped and started at will.
    Activate(True)              starts the image cycling.
    Activate(True, delay=n)     starts the image cycling at delay Seconds in the future
    Activate(False)             immediately stop cycling at the current image.
    Activate(False, delay=n)    stop cycling at delay Seconds in the future and Reset

    Reset()                     immediately stop cycling and set the image to the first supplied image.
    StopAt(index)               stop cycling when the image reaches the index.


The time period for cycling can be set to whatever you define in milliseconds and is changeable during operation.

All supplied bitmaps are scaled to the same size as:
    the size parameter if supplied;
    the size of the smallest bitmap in the series.
    Any subsequent Additions or Extensions to the series are scaled to the same size.

wx.NullBitmap can used as a valid placeholder or visual gap in the series.

The current image index can be queried or Set

Responds to Mouse events like a normal StaticBitmap


Author:     J Healey
Created:    19/04/2023
Copyright:  J Healey - 2023
License:    GPL 3 or any later version
Version:    1.2.0
Email:      <rolfofsaxony@gmx.com>
Written & tested: Linux, Python 3.8.10, wxPython 4.2.0, Pillow versions 9.1.0 and 9.5.0

Changelog:
1.2.0   Add the PIL image library as it processes images up to 50 times quicker than wx.Image,
         depending on image size and the number of images to process
        The PIL library also seems to cope with 'optimised' Gif files and animated .Webp files

        Include thread for rescaling the images, as this allows us to start displaying the images,
        as soon as the first one is ready.
        (ActiveBitmap will cycle through available images, adding to the animation as more images become available)

        The PIL library remains optional. If it is not available ActiveBitmap reverts to wx.Image.

        The test for a single image at Activate() checks to ensure that the Scaling thread isn't running,
         which would signify that more images are currently be produced.

        Include File Error bitmaps in case something goes wrong during Gif conversion.

1.1.0   Ensure that illegal values cannot be set in the SetRange function
        Test for missing bitmap_list, allowing easier definition of a dynamic resizing bitmap,
         rather than an animation
        Don't start the period timer is there is only a single bitmap.

    Default class constructor.

        @param parent:  Parent window. Must not be None.
        @param id:      identifier. A value of -1 indicates a default value.
        @param bitmap:  Primary bitmap to use
        @param pos:     Position. If the position (-1, -1) is specified
                        then a default position is chosen.
        @param size:    If the default size (-1, -1) is specified then the size is is based on the size of
                        the primary bitmap.
        @param style:   wx.Border style default wx.TRANSPARENT_WINDOW|wx.BORDER_NONE
        @param bitmap_list:
                        A list containing zero or more wx.Bitmap entries that will by cycled through when activated
                         wx.NullBitmap is a valid entry
                         A single Gif filename is a valid entry
        @param name:    Widget name.

    Functions:

        Activate(boolean, delay=n, repetitions=-1)
                                    True Start image cycling
                                    False Stop image cycling
                                    delay will Start or Stop at delay Seconds in the future depending on the boolean
                                    repetitions, the number times to play the sequence, -1 equals forever
                                   Defaults: True , -1, -1

        SetTimerPeriod(period=n)    Wait Period in milliseconds before cycling to next image
                                    If this is used whilst the timer is currently running, it is
                                     applied instantly allowing you to speed up or slow down the
                                     current rate of cycling.
                                    Default: 1000

        GetTimerPeriod()            Get current Wait Period in milliseconds

        Reset()                     Stop cycling and revert image to primary image

        StopAt(index=0)             Stop cycling when image next reaches this index

        Stop()                      Alternate to Activate(False)

        Start()                     Alternate to Activate(True)

        GetIndex()                  Return current image index

        SetIndex(index=0)           Set current image index

        Extend(extension_list=[])   Extend current list of images with a list of extra wx.Bitmaps

        InsertBitmap(index=0, bitmap=wx.NullBitmap)
                                    Insert a wx.Bitmap at index

        DeleteBitmap(index=0)       Delete bitmap at position index

        GetCount()                  Return the number of images currently in use

        SetRange(start=0, end=999)  Set a sub-set of images to cycle through

        GetRange()                  Returns the start and end of the current cycle range

        GetFrameBitmap(index)       Return indexed image as a wx.Bitmap

        GetFrameImage(index)        Return indexed image as a wx.Image

        IsRunning()                 Return True or False if the control is Active

    Converting a GIF file to an ActiveBitmap
    ----------------------------------------
    The speed of the animation is governed by period timer, irrespective of the original speed of the gif file.

    Method 1
    --------
    Allow ActiveBitmap to extract the images from the Gif file
    When creating the control, specify the bitmap to be a wx.NullBitmap and then give the Gif filename as the only entry
     in the bitmap_list
    e.g.
        Abitmap = ActiveBitmap(panel, -1, bitmap=wx.NullBitmap, bitmap_list = ['led1s.gif',])

    Method 2 extract the Gif file images manually, then pass them as a list for PIL see the demonstration code
    --------
        gif_list = []
        f = 'led1s.gif'
        cnt = wx.Image.GetImageCount(f)
        im = wx.Image()
        for i in range(cnt):
            im.LoadFile(f, index=i)
            bmp = im.ConvertToBitmap(depth=32)
            im.Clear()
            bmp.UseAlpha()
            gif_list.append(bmp)
        if not gif_list: # ensure at least 1 image if there's a problem
            gif_list.append(wx.NullBitmap)

    The list 'gif_list' can now be set as the bitmap_list and the initial bitmap set to gif_list[0]
    Speed of loading can be an issue if there are many images in the gif.
    Modifying the options in range() you could convert just every 3rd image i.e. for i in range(0, count, 3)
     or a group of images in the middle e.g. for i in range(20, 50)
     or reverse the images e.g. for i in range(60, 20, -1)

    When creating the control, specify the bitmap to be the first entry in the gif_list and then specify the gif_list
     as the bitmap_list
    e.g. Abitmap = ActiveBitmap(panel, -1, bitmap=gif_list[0], bitmap_list = gif_list)

    NOTE:
    Gif files can be optimised i.e. they aren't strictly a batch of images of the same size.
    Optimisation is often acheived by creating frame images that contain only the differences in pixels from the previous frame.
    Obviously this can reduce the file size immensely but to play the animation, you need a more sophisticated tool
     than ActiveBitmap e.g. wx.adv.Animation and wx.adv.AnimationCtrl
    imagemagick's 'identify' command will reveal this, as the individual image sizes will differ or be the same.
    If they differ don't use ActiveBitmap OR grab a copy of a program called gifsicle, which has an unoptimise option
        gifsicle --unoptimize -i optimised.gif > unoptimised.gif
    and use the unoptimised output.


Example program

import wx
from ActiveBitmap import ActiveBitmap
class Frame(wx.Frame):
    def __init__(self, parent):
        wx.Frame.__init__(self, parent, -1, "ActiveBitmap Demonstration", size=(320, 450))
        panel = wx.Panel(self)
        box = wx.StaticBox(panel, wx.ID_ANY, "Animations", pos=(10, 10), size=(300, 400))
        bitmap1 = ActiveBitmap(box, -1, pos=(10, 10), bitmap=wx.NullBitmap, bitmap_list=['cube_s.gif',])
        bitmap1.SetTimerPeriod(50)

        bitmap2 = ActiveBitmap(box, -1, pos=(10, 150), bitmap=wx.Bitmap('amber.png'), \
                               bitmap_list=[wx.Bitmap('red.png'), wx.Bitmap('green.png'),])
        bitmap2.SetTimerPeriod(250)

        bitmap3 = ActiveBitmap(box, -1, pos=(10, 200), size=(200, -1), bitmap=wx.NullBitmap, bitmap_list=['led1s.gif',])
        bitmap3.SetTimerPeriod(50)

        self.Show()

        bitmap1.Activate()
        bitmap2.Activate()
        bitmap3.Activate()

app = wx.App()
frame = Frame(None)
app.MainLoop()


'''
import wx
import imghdr
import threading
try:
    from PIL import Image, ImageSequence
    pil_loaded = True
except Exception:
    pil_loaded = False

#-------------------------------------------------------------------------------

class GradientPanel(wx.Panel):
    def __init__(self, parent, initialColour="#ffffff", destColour="#7f7f7f", singleGradient=False, \
                 direction=wx.SOUTH, inherit=False):
        wx.Panel.__init__(self, parent)

        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        self.Bind(wx.EVT_SIZE, self.OnResize)

        if inherit:
            self.initial = parent.GetBackgroundColour()
            self.dest = wx.Colour(self.initial).ChangeLightness(170)
        else:
            try:
                self.initial = wx.Colour(initialColour)
            except AssertionError:
                self.initial = wx.Colour("#ffffff")
            try:
                self.dest = wx.Colour(destColour)
            except AssertionError:
                self.dest = wx.Colour("#7f7f7f")

        if singleGradient: # test initial colour to decide gradient lighter or darker
            if self.initial.RGB > wx.Colour("#7f7f7f").RGB:
                chg = 60
            else:
                chg = 170
            self.dest = wx.Colour(self.initial).ChangeLightness(chg)

        self.dir = direction
        #------------

        # Simplified init method.
        self.SetProperties()

    #---------------------------------------------------------------------------

    def SetProperties(self):
        if wx.Platform == "__WXMSW__":
            self.SetDoubleBuffered(True)  # No flicker

    def OnResize(self, event):
        self.Refresh()
        event.Skip()

    def OnEraseBackground(self, event):
        dc = event.GetDC()
        dc.GradientFillLinear(self.GetClientRect(),
                              self.initial, self.dest,
                              self.dir)
        #event.Skip() # don't use

#-------------------------------------------------------------------------------

class ActiveBitmap(wx.StaticBitmap):
    def __init__(self, parent, id=wx.ID_ANY, bitmap=wx.NullBitmap, pos=wx.DefaultPosition,
             size=wx.DefaultSize, style=wx.BORDER_NONE|wx.TRANSPARENT_WINDOW, bitmap_list=None, name=''):
        wx.StaticBitmap.__init__(self, parent, id, bitmap, pos, size, style, name)

        self.bitmaps = []
        self.bitmap_index = 0
        if not bitmap_list:
            self.bitmap_list = []
        else:
            self.bitmap_list = bitmap_list
        self._size = size
        self._stop_at = -1
        self.range_start = 0
        self.range_end = 999
        self.repetitions = -1
        self.timer_period = 1000 # default timer period = 1 second
        self.name = name

        bsize = self.SizeBitmap(bitmap)

        # perform scaling via thread allows for starting showing bitmaps as soon as the first one is ready
        self.thread = ScaleThread(self, bsize)

        while not self.bitmaps:
            wx.MilliSleep(100)
        self.bitmap = self.bitmaps[0]
        wx.StaticBitmap.SetBitmap(self, self.bitmap)

        self.timer = wx.Timer(self)
        self.delay_start_timer = wx.Timer(self)
        self.delay_stop_timer = wx.Timer(self)

        self.Bind(wx.EVT_TIMER, self.OnNext, self.timer)
        self.Bind(wx.EVT_TIMER, lambda event: self.OnDelay(event, True), self.delay_start_timer)
        self.Bind(wx.EVT_TIMER, lambda event: self.OnDelay(event, False), self.delay_stop_timer)

    def SizeBitmap(self, initial_bitmap):
        '''
        In case a wx.NullBitmap is passed as the initial bitmap, hunt for the first valid bitmap to ascertain size
        Also validate bitmap_list in case of wx.NullBitmap or rebuild bitmap list if it's a Gif file
        '''
        valid_bmp = wx.Bitmap.FromRGBA(1, 1, red=255, green=255, blue=255, alpha=0)
        fb = ErrorBitmap()
        failed_bmps = [fb.failed1_bmp, fb.failed2_bmp]
        w, h = self._size
        sizes = [(w, h),]

        # is the bitmap list a gif file if so generate a new bitmap_list from it.
        if self.bitmap_list:
            try:
                f = self.bitmap_list[0]
            except Exception:
                f = None
        else:
            f = None
        try:
            ftype = imghdr.what(f)
        except FileNotFoundError:
            ftype = None
            self.bitmap_list = list(failed_bmps)
            w = 100
            h = 30
        except Exception as e:
            ftype = None

        if ftype in ['gif','webp']:
            self.bitmap_list = []
            # ŧest for the PIL library as it's up to 50 times quicker in comparison and handles optimised Gif files
            if pil_loaded:
                pil_scaled = False
                if w >=0 and h >=0:
                    pil_scaled = True
                im = Image.open(f)
                seq = ImageSequence.all_frames(im)
                for frame in seq:
                    if pil_scaled:
                        frame = frame.resize((w, h))
                    bmp = wx.Bitmap.FromBufferRGBA(frame.width, frame.height, frame.convert("RGBA").tobytes())
                    if bmp.HasAlpha():
                        bmp.UseAlpha()
                    self.bitmap_list.append(bmp)
            else:
                im = wx.Image()
                im.SetOption(wx.IMAGE_OPTION_GIF_TRANSPARENCY, wx.IMAGE_OPTION_GIF_TRANSPARENCY_UNCHANGED)
                cnt = wx.Image.GetImageCount(f)
                for i in range(cnt):
                    im.LoadFile(f, type=wx.BITMAP_TYPE_GIF, index=i)
                    bmp = im.ConvertToBitmap(depth=32)
                    im.Clear()
                    if bmp.HasAlpha():
                        bmp.UseAlpha()
                    self.bitmap_list.append(bmp)
        else:
            # insert the initial bitmap at the front of the list for sizing
            if initial_bitmap != wx.NullBitmap:
                self.bitmap_list.insert(0, initial_bitmap)

        if not self.bitmap_list: # ensure at least 1 image if there's a problem
            self.bitmap_list.append(wx.Bitmap(fb.failed1_bmp))

        bsize = (-1, -1)
        # validate bitmap list
        for i, b in enumerate(self.bitmap_list):
            try:
                bsize = b.GetSize()
                sizes.append((bsize.x, bsize.y))
            except AssertionError:
                self.bitmap_list[i] = valid_bmp

        # find smallest tuple where <=1 is absent
        try:
            bsize = min(y for y in sizes if y[0] > 1 and y[1] > 1)
        except Exception:
            bsize = (1, 1)

        # Calculate the dimension ratio in case the supplied size parameter is missing a dimension e.g. (30, -1)
        if self._size != wx.DefaultSize:
            if w <=0 or h <=0:
                ratio = bsize[1] / bsize[0]
                if h < 0 and w >= 0:
                    h = int(w * ratio)
                if w < 0 and h >= 0:
                    w = int(h / ratio)

        if w >=0 and h >=0:
            bsize = (w, h)
        self.SetMinSize(bsize)
        self.SetSize(bsize)
        return bsize

    def Activate(self, arg=True, delay=-1, repetitions=-1):
        if len(self.bitmaps) < 2 and not self.thread.is_alive(): # single image it's a staticbitmap don't cycle
            if self.timer.IsRunning():
                self.timer.Stop()
            return
        if delay > 0: # Delayed activation
            if arg:
                self.delay_start_timer.StartOnce(delay * 1000)
            else:
                self.delay_stop_timer.StartOnce(delay * 1000)
            return

        if arg is True:
            self.bitmap_index = self.range_start
            self.timer.Start(self.timer_period)
        else:
            if self.timer.IsRunning():
                self.timer.Stop()
        self.repetitions = repetitions

    def OnDelay(self, event, arg):
        if arg:
            self.Activate(arg)
        else:
            self.Reset()

    def OnNext(self, event=None):
        self.bitmap = self.bitmaps[self.bitmap_index]
        wx.StaticBitmap.SetBitmap(self, self.bitmap)
        if self._stop_at > -1 and self._stop_at == self.bitmap_index:
            self._stop_at = -1
            self.timer.Stop()
        self.Refresh()
        self.bitmap_index += 1
        if self.bitmap_index >= len(self.bitmaps) and self.repetitions == 0:
            self.Reset()
        if self.bitmap_index >= len(self.bitmaps) or self.bitmap_index >= self.range_end:
            self.repetitions -= 1
            self.bitmap_index = self.range_start

    def SetTimerPeriod(self, period=1000):
        self.timer_period = period
        if self.timer.IsRunning():
            self.timer.Start(self.timer_period)

    def GetTimerPeriod(self):
        return self.timer_period

    def Reset(self):
        if self.timer.IsRunning():
            self.timer.Stop()
        wx.StaticBitmap.SetBitmap(self, self.bitmaps[0])
        self.bitmap_index = len(self.bitmaps) - 1
        self.Refresh()

    def StopAt(self, index=0):
        self._stop_at = index

    def Stop(self):
        if self.timer.IsRunning():
            self.timer.Stop()

    def Start(self):
        self.bitmap_index = self.range_start
        self.timer.Start(self.timer_period)

    def GetIndex(self):
        return self.bitmap_index

    def GetCount(self):
        return len(self.bitmaps)

    def GetFrameBitmap(self, index=-1):
        if index >= 0 and index < len(self.bitmaps) - 1:
            return self.bitmaps[index]

    def GetFrameImage(self, index=-1):
        if index >= 0 and index < len(self.bitmaps) - 1:
            return self.bitmaps[index].ConvertToImage()

    def SetRange(self, start=0, end=999):
        if start >= 0 and start <= len(self.bitmaps) - 1:
            self.range_start = start
        else:
            self.range_start = 0
        if end >= self.range_start and end <= len(self.bitmaps) - 1:
            self.range_end = end
        else:
            self.range_end = len(self.bitmaps) - 1

    def GetRange(self):
        return self.range_start, self.range_end

    def SetIndex(self, index=0):
        self.bitmap_index = index
        if self.bitmap_index >= len(self.bitmaps) or self.bitmap_index >= self.range_end:
            self.bitmap_index = self.range_start

    def Extend(self, extension_list = []):
        if len(self.bitmaps): # Check if at least one image is available
            bsize = self.bitmaps[0].GetSize()
        else:
            bsize = self.GetMinSize()

        for b in extension_list:
            img = b.ConvertToImage()
            img.Rescale(bsize[0], bsize[1], quality=wx.IMAGE_QUALITY_HIGH)
            bmp = img.ConvertToBitmap(depth=32)
            bmp.UseAlpha()
            self.bitmaps.append(bmp)

    def InsertBitmap(self, index=0, bitmap=wx.NullBitmap):
        bsize = self.bitmaps[0].GetSize()
        img = bitmap.ConvertToImage()
        img.Rescale(bsize[0], bsize[1], quality=wx.IMAGE_QUALITY_HIGH)
        bmp = img.ConvertToBitmap(depth=32)
        bmp.UseAlpha()
        self.bitmaps.insert(index, bmp)

    def DeleteBitmap(self, index=0):
        if index <= len(self.bitmaps) -1:
            x = self.bitmaps.pop(index)

    def IsRunning(self):
        return self.timer.IsRunning()

    def DoGetBestSize(self):
        return self.bitmap.GetSize()

    def Enable(self, arg=True):
        wx.StaticBitmap.Enable(self, arg)
        self.Refresh()

    def Disable(self, arg=False):
        wx.StaticBitmap.Enable(self, arg)
        self.Refresh()

    def OnEraseBackground(self,event):
        pass

    def OnSize(self, event):
        event.Skip()
        self.Refresh()

    def SetWindowStyle(self, style):
        self._style = style
        self.Refresh()

class ScaleThread(threading.Thread):
    '''
    Scale bitmaps via a thread for speed up
    '''
    def __init__(self,parent_target, bsize):
        threading.Thread.__init__(self)
        self.parent = parent_target
        self.bitmaps = self.parent.bitmaps
        self.bitmap_list = self.parent.bitmap_list
        self.bsize = bsize
        self.start()  # start the thread

    def run(self):
        # Resize bitmaps to one standard size
        for b in self.bitmap_list:
            img = b.ConvertToImage()
            img.Rescale(self.bsize[0], self.bsize[1], quality=wx.IMAGE_QUALITY_HIGH)
            bmp = img.ConvertToBitmap(depth=32)
            if bmp.HasAlpha():
                bmp.UseAlpha()
            self.bitmaps.append(bmp)
        return

class ErrorBitmap():
    '''
    Create a bitmap of text for use if there's a file error
    '''
    def __init__(self):
        self.failed1_bmp = wx.Bitmap(100, 30)
        dc = wx.MemoryDC()
        dc.SelectObject(self.failed1_bmp)
        dc.Clear()
        text = "File Error"
        tw, th = dc.GetTextExtent(text)
        dc.DrawText(text, int((100-tw)/2),  int((30-th)/2))
        dc.SelectObject(wx.NullBitmap)
        self.failed2_bmp = wx.Bitmap(100, 30)
        dc = wx.MemoryDC()
        dc.SelectObject(self.failed2_bmp)
        dc.Clear()
        dc.DrawText(text, int((100-tw)/2)+2,  int((30-th)/2))
        dc.SelectObject(wx.NullBitmap)

class Demonstration(wx.Frame):
    import wx.lib.scrolledpanel as scrolled
    def __init__(self):
        wx.Frame.__init__(self, None, -1, title='Active Bitmap/Animation Demonstration', size=(500, 300))

        fb = ErrorBitmap()
        failed_bmps = [fb.failed1_bmp, fb.failed2_bmp]

        # gif preload example
        # loading the gif images can be slow so if possible arrange to do this before hand
        # or dispense with this and let ActiveBitmap extract the images from the gif for you

        gif_list = []
        #f = 'cube_s.gif'   # pre-extract this Gif file for bitmap1 below
        #f = 'led1s.gif'    # pre-extract this Gif file for bitmap1 below
        f = None            # Don't pre-extract a Gif file for bitmap1 below
                            #  Either it will be normal image files or ActiveBitmap will extract
                            #  the Gif file for you depending on the definition of bitmap1 below

        # Optional Pre-extraction routine
        try:
            ftype = imghdr.what(f)
        except Exception:
            ftype = None
            print("No Gif file to preload")
        if ftype == 'gif':
            print("Pre-loading", f)
            # ŧest for the PIL library as it's 50 times quicker in comparison and handles optimised Gif files
            if pil_loaded:
                im = Image.open(f)
                seq = ImageSequence.all_frames(im)
                for frame in seq:
                    bmp = wx.Bitmap.FromBufferRGBA(frame.width, frame.height, frame.convert("RGBA").tobytes())
                    if bmp.HasAlpha():
                        bmp.UseAlpha()
                    gif_list.append(bmp)
            else:
                im = wx.Image()
                im.SetOption(wx.IMAGE_OPTION_GIF_TRANSPARENCY, wx.IMAGE_OPTION_GIF_TRANSPARENCY_UNCHANGED)
                cnt = wx.Image.GetImageCount(f)
                for i in range(cnt):
                    im.LoadFile(f, type=wx.BITMAP_TYPE_GIF, index=i)
                    bmp = im.ConvertToBitmap(depth=32)
                    im.Clear()
                    if bmp.HasAlpha():
                        bmp.UseAlpha()
                    gif_list.append(bmp)

        if not gif_list: # ensure at least 1 image if there's a problem
            gif_list.extend(failed_bmps)

        # Pre-extraction routine ends

        panel = Demonstration.scrolled.ScrolledPanel(self, -1, style=wx.TAB_TRAVERSAL)
        panel.SetupScrolling()
        panel.SetBackgroundColour("#e6e6fa") # lavender
        #panel = GradientPanel(self, initialColour="#30f030", destColour="#ff3030", \
        #                      inherit=False, singleGradient=False, direction=wx.SOUTH)

        self.text = wx.StaticText(panel, -1, label="My animated bitmaps")
        self.text.SetForegroundColour(wx.BLUE)

        # standard NON Gif example a list of image files converted to wx.Bitmaps
        #self.bitmap1 = ActiveBitmap(panel, -1, bitmap=wx.Bitmap('1.png'), size=(30, 30), name="Bitmap 1", \
        #                       bitmap_list = [wx.Bitmap('2.png'), wx.Bitmap('3.png'), wx.Bitmap('4.png'),])

        # gif example of a pre-extracted Gif ( see above )
        #self.bitmap1 = ActiveBitmap(panel, -1, bitmap=gif_list[0], bitmap_list = gif_list, name="Gif_preloaded")

        # straight Gif example passes the Gif filename as the lone argument to bitmap_list
        self.bitmap1 = ActiveBitmap(panel, -1, bitmap=wx.NullBitmap, bitmap_list = ['hourglass.gif',], size=(90, 100), name="Hourglass")

        self.bitmap2 = ActiveBitmap(panel, -1, bitmap=wx.Bitmap('amber.png'), name="Bitmaps", \
                               bitmap_list = [wx.Bitmap('red.png'), wx.Bitmap('green.png'),])

        self.bitmap3 = ActiveBitmap(panel, -1, bitmap=wx.NullBitmap, bitmap_list=['load1.gif',], name="Load")
        self.bitmap3.SetTimerPeriod(200)

        buttonReset = wx.Button(panel, -1, label="Reset")
        buttonActivate = wx.Button(panel, -1, label="Activate")
        buttonStop = wx.Button(panel, -1, label="Stop")

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.text, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 20)
        sizer.Add(self.bitmap1, 0, wx.ALIGN_CENTER_VERTICAL)

        sizer2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer2.Add(self.bitmap2, 0, wx.ALIGN_CENTER_VERTICAL)

        sizer3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer3.Add(self.bitmap3, 0, wx.ALIGN_CENTER_VERTICAL)

        buttonsizer = wx.BoxSizer(wx.HORIZONTAL)

        buttonsizer.Add(buttonReset)
        buttonsizer.Add(buttonActivate)
        buttonsizer.Add(buttonStop)

        mainsizer = wx.BoxSizer(wx.VERTICAL)
        mainsizer.Add(sizer, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        mainsizer.Add(sizer2, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        mainsizer.Add(sizer3, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        mainsizer.Add(buttonsizer, 0, wx.ALIGN_CENTER|wx.ALL, 10)

        buttonReset.Bind(wx.EVT_BUTTON, self.OnReset)
        buttonActivate.Bind(wx.EVT_BUTTON, self.OnActivate)
        buttonStop.Bind(wx.EVT_BUTTON, self.OnStop)

        # ActiveBitmaps are clickable
        self.bitmap1.Bind(wx.EVT_LEFT_DOWN, self.OnClick)
        self.bitmap2.Bind(wx.EVT_LEFT_DOWN, self.OnClick)
        self.bitmap3.Bind(wx.EVT_LEFT_DOWN, self.OnClick)

        panel.SetSizer(mainsizer)

        # timer spèed if playing the gif file
        self.bitmap1.SetTimerPeriod(200)
        # standard timer speed
        #self.bitmap1.SetTimerPeriod(500)

        #self.bitmap1.SetRange(50, 80) # Animate a sub-set of the loaded images from image 50 to image 80

        self.bitmap1.Activate(True, delay=0, repetitions=-1)

        #self.bitmap1.Activate(True, repetitions=3) # Start animation and Stop after 3 repetitions
        #self.bitmap1.Activate(True, delay=10) # Start animation after 10 seconds
        #self.bitmap1.Activate(False, delay=10) # Stop animation after 10 seconds
        #self.bitmap1.StopAt(30) # Stop animation at image 30

        self.bitmap2.SetTimerPeriod(500)
        self.bitmap2.Activate(True) # Start immediately
        self.bitmap2.Activate(False, delay=20) # Stop animation after 20 seconds
        self.bitmap2.SetToolTip("Cycles Amber, Red and Green for 20 seconds")

        self.bitmap3.Activate()

        self.Show()

    def OnReset(self, event):
        self.bitmap1.Reset()
        self.text.SetLabel("Reset and Stopped")
        # Various Testing options
        #for i in range(0, self.bitmap1.GetCount()):
        #    self.bitmap1.DeleteBitmap(index=0)
        #self.bitmap1.Extend([wx.Bitmap('./cube1.jpg'), wx.Bitmap('./cube2.jpg'), wx.Bitmap('./cube3.jpg'),])
        #self.bitmap1.SetTimerPeriod(int(self.bitmap1.GetTimerPeriod()/2)) # double the speed of the animation
        #self.bitmap1.Extend([wx.Bitmap('5.png'),])
        #self.bitmap1.DeleteBitmap(index=1)
        #self.bitmap1.InsertBitmap(index=1, bitmap=wx.Bitmap('5.png'))

    def OnActivate(self, event):
        self.bitmap1.Activate(True)
        self.bitmap2.Activate(True) # Start immediately
        self.bitmap2.Activate(False, delay=20) # Stop animation after 20 seconds
        self.text.SetLabel("My animated bitmaps")

    def OnStop(self, event):
        self.bitmap1.Activate(False)
        self.text.SetLabel("Stopped")

    def OnClick(self, event):
        obj = event.GetEventObject()
        print(event.GetClassName(), obj.name, "Mouse Button", event.GetButton(), "Clicked")
        print("Running", obj.IsRunning(), "at frame", obj.GetIndex(), "of", obj.GetCount(), "\n")


if __name__ == '__main__':
    app = wx.App()
    frame = Demonstration()
    app.MainLoop()
